@@ -1,48 +1,15 @@
 package SQL::Abstract::Test; # see doc at end of file
 
+use SQL::Abstract::_TempExtlib;
+
 use strict;
 use warnings;
 use base qw(Test::Builder::Module Exporter);
+use Data::Dumper;
 use Test::Builder;
 use Test::Deep ();
 use SQL::Abstract::Tree;
 
-{
-  my $class;
-  if ($class = $ENV{SQL_ABSTRACT_TEST_AGAINST}) {
-    my $mod = join('/', split '::', $class).".pm";
-    require $mod;
-    eval qq{sub SQL::Abstract () { "\Q${class}\E" }; 1}
-      or die "Failed to create const sub for ${class}: $@";
-  }
-  if ($ENV{SQL_ABSTRACT_TEST_EXPAND_STABILITY}) {
-    $class ||= do { require SQL::Abstract; 'SQL::Abstract' };
-    my $orig = $class->can('expand_expr');
-    require Data::Dumper::Concise;
-    my $wrapped = sub {
-      my ($self, @args) = @_;
-      my $e1 = $self->$orig(@args);
-      return $e1 if our $Stab_Check_Rec;
-      local $Stab_Check_Rec = 1;
-      my $e2 = $self->$orig($e1);
-      my ($d1, $d2) = map Data::Dumper::Concise::Dumper($_), $e1, $e2;
-      (our $tb)->is_eq(
-        $d2, $d1,
-        'expand_expr stability ok'
-      ) or do {
-        require Path::Tiny;
-        Path::Tiny->new('e1')->spew($d1);
-        Path::Tiny->new('e2')->spew($d2);
-        system('diff -u e1 e2 1>&2');
-        die "Differences between e1 and e2, bailing out";
-      };
-      return $e1;
-    };
-    no strict 'refs'; no warnings 'redefine';
-    *{"${class}::expand_expr"} = $wrapped;
-  }
-}
-
 our @EXPORT_OK = qw(
   is_same_sql_bind is_same_sql is_same_bind
   eq_sql_bind eq_sql eq_bind dumper diag_where
@@ -58,30 +25,8 @@ our $order_by_asc_significant = 0;
 our $sql_differ; # keeps track of differing portion between SQLs
 our $tb = __PACKAGE__->builder;
 
-sub _unpack_arrayrefref {
-
-  my @args;
-  for (1,2) {
-    my $chunk = shift @_;
-
-    if (ref $chunk eq 'REF' and ref $$chunk eq 'ARRAY') {
-      my ($sql, @bind) = @$$chunk;
-      push @args, ($sql, \@bind);
-    }
-    else {
-      push @args, $chunk, shift @_;
-    }
-
-  }
-
-  # maybe $msg and ... stuff
-  push @args, @_;
-
-  @args;
-}
-
 sub is_same_sql_bind {
-  my ($sql1, $bind_ref1, $sql2, $bind_ref2, $msg) = &_unpack_arrayrefref;
+  my ($sql1, $bind_ref1, $sql2, $bind_ref2, $msg) = @_;
 
   # compare
   my $same_sql  = eq_sql($sql1, $sql2);
@@ -106,7 +51,7 @@ sub is_same_sql {
   my ($sql1, $sql2, $msg) = @_;
 
   # compare
-  my $same_sql = eq_sql($sql1, $sql2);
+  my $same_sql  = eq_sql($sql1, $sql2);
 
   # call Test::Builder::ok
   my $ret = $tb->ok($same_sql, $msg);
@@ -139,45 +84,36 @@ sub is_same_bind {
 }
 
 sub dumper {
-  # FIXME
-  # if we save the instance, we will end up with $VARx references
-  # no time to figure out how to avoid this (Deepcopy is *not* an option)
-  require Data::Dumper;
-  Data::Dumper->new([])->Terse(1)->Indent(1)->Useqq(1)->Deparse(1)->Quotekeys(0)->Sortkeys(1)->Maxdepth(0)
-    ->Values([@_])->Dump;
+  Data::Dumper->new([])->Terse(1)->Indent(1)->Useqq(1)->Deparse(1)->Quotekeys(0)->Sortkeys(1)->Maxdepth(0)->Values([@_])->Dump;
 }
 
 sub diag_where{
-  $tb->diag("Search term:\n" . &dumper);
+  $tb->diag( "Search term:\n" . &dumper );
 }
 
 sub _sql_differ_diag {
-  my $sql1 = shift || '';
-  my $sql2 = shift || '';
+  my ($sql1, $sql2) = @_;
 
-  if (my $profile = $ENV{SQL_ABSTRACT_TEST_TREE_PROFILE}) {
-    my $sqlat = SQL::Abstract::Tree->new(profile => $profile);
-    $_ = $sqlat->format($_) for ($sql1, $sql2);
-  }
-
-  $tb->${\($tb->in_todo ? 'note' : 'diag')} (
+  $tb->${\( $tb->in_todo ? 'note' : 'diag')} (
        "SQL expressions differ\n"
-      ." got: $sql1\n"
-      ."want: $sql2\n"
-      ."\nmismatch around\n$sql_differ\n"
+      ."     got: $sql1\n"
+      ."expected: $sql2\n"
+      ."differing in :\n$sql_differ\n"
   );
 }
 
 sub _bind_differ_diag {
   my ($bind_ref1, $bind_ref2) = @_;
 
-  $tb->${\($tb->in_todo ? 'note' : 'diag')} (
-    "BIND values differ " . dumper({ got => $bind_ref1, want => $bind_ref2 })
-  );
+  $tb->${\( $tb->in_todo ? 'note' : 'diag')} (
+       "BIND values differ\n"
+      ."     got: " . dumper($bind_ref1)
+      ."expected: " . dumper($bind_ref2)
+      );
 }
 
 sub eq_sql_bind {
-  my ($sql1, $bind_ref1, $sql2, $bind_ref2) = &_unpack_arrayrefref;
+  my ($sql1, $bind_ref1, $sql2, $bind_ref2) = @_;
 
   return eq_sql($sql1, $sql2) && eq_bind($bind_ref1, $bind_ref2);
 }
@@ -200,8 +136,8 @@ sub _eq_sql {
   my ($left, $right) = @_;
 
   # one is defined the other not
-  if ((defined $left) xor (defined $right)) {
-    $sql_differ = sprintf ("[%s] != [%s]\n", map { defined $_ ? $sqlat->unparse($_) : 'N/A' } ($left, $right) );
+  if ( (defined $left) xor (defined $right) ) {
+    $sql_differ = sprintf ("[%s] != [%s]\n", map { defined $_ ? $sqlat->unparse ($_) : 'N/A' } ($left, $right) );
     return 0;
   }
 
@@ -217,14 +153,14 @@ sub _eq_sql {
 
   # one is empty
   if (@$left == 0 or @$right == 0) {
-    $sql_differ = sprintf ("left: %s\nright: %s\n", map { @$_ ? $sqlat->unparse($_) : 'N/A'} ($left, $right) );
+    $sql_differ = sprintf ("left: %s\nright: %s\n", map { @$_ ? $sqlat->unparse ($_) : 'N/A'} ($left, $right) );
     return 0;
   }
 
   # one is a list, the other is an op with a list
   elsif (ref $left->[0] xor ref $right->[0]) {
     $sql_differ = sprintf ("[%s] != [%s]\nleft: %s\nright: %s\n", map
-      { ref $_ ? $sqlat->unparse($_) : $_ }
+      { ref $_ ? $sqlat->unparse ($_) : $_ }
       ($left->[0], $right->[0], $left, $right)
     );
     return 0;
@@ -237,7 +173,7 @@ sub _eq_sql {
         if (! $sql_differ or $sql_differ !~ /left\:\s .+ right:\s/xs) {
           $sql_differ ||= '';
           $sql_differ .= "\n" unless $sql_differ =~ /\n\z/;
-          $sql_differ .= sprintf ("left: %s\nright: %s\n", map { $sqlat->unparse($_) } ($left, $right) );
+          $sql_differ .= sprintf ("left: %s\nright: %s\n", map { $sqlat->unparse ($_) } ($left, $right) );
         }
         return 0;
       }
@@ -249,7 +185,7 @@ sub _eq_sql {
   else {
 
     # unroll parenthesis if possible/allowed
-    unless ($parenthesis_significant) {
+    unless ( $parenthesis_significant ) {
       $sqlat->_parenthesis_unroll($_) for $left, $right;
     }
 
@@ -258,7 +194,7 @@ sub _eq_sql {
       $sqlat->_strip_asc_from_order_by($_) for $left, $right;
     }
 
-    if ($left->[0] ne $right->[0]) {
+    if ( $left->[0] ne $right->[0] ) {
       $sql_differ = sprintf "OP [$left->[0]] != [$right->[0]] in\nleft: %s\nright: %s\n",
         $sqlat->unparse($left),
         $sqlat->unparse($right)
@@ -278,7 +214,7 @@ sub _eq_sql {
     # if operators are identical, compare operands
     else {
       my $eq = _eq_sql($left->[1], $right->[1]);
-      $sql_differ ||= sprintf ("left: %s\nright: %s\n", map { $sqlat->unparse($_) } ($left, $right) ) if not $eq;
+      $sql_differ ||= sprintf ("left: %s\nright: %s\n", map { $sqlat->unparse ($_) } ($left, $right) ) if not $eq;
       return $eq;
     }
   }
@@ -333,83 +269,52 @@ B<Disclaimer> : the semantic equivalence handling is pretty limited.
 A lot of effort goes into distinguishing significant from
 non-significant parenthesis, including AND/OR operator associativity.
 Currently this module does not support commutativity and more
-intelligent transformations like L<De Morgan's laws
-|http://en.wikipedia.org/wiki/De_Morgan's_laws>, etc.
+intelligent transformations like Morgan laws, etc.
 
-For a good overview of what this test framework is currently capable of refer
+For a good overview of what this test framework is capable of refer
 to C<t/10test.t>
 
 =head1 FUNCTIONS
 
 =head2 is_same_sql_bind
 
-  is_same_sql_bind(
-    $given_sql, \@given_bind,
-    $expected_sql, \@expected_bind,
-    $test_msg
-  );
+  is_same_sql_bind($given_sql,    \@given_bind,
+                   $expected_sql, \@expected_bind, $test_msg);
 
-  is_same_sql_bind(
-    \[$given_sql, @given_bind],
-    \[$expected_sql, @expected_bind],
-    $test_msg
-  );
-
-  is_same_sql_bind(
-    $dbic_rs->as_query
-    $expected_sql, \@expected_bind,
-    $test_msg
-  );
-
-Compares given and expected pairs of C<($sql, \@bind)> by unpacking C<@_>
-as shown in the examples above and passing the arguments to L</eq_sql> and
-L</eq_bind>. Calls L<Test::Builder/ok> with the combined result, with
-C<$test_msg> as message.
-If the test fails, a detailed diagnostic is printed.
+Compares given and expected pairs of C<($sql, \@bind)>, and calls
+L<Test::Builder/ok> on the result, with C<$test_msg> as message. If the test
+fails, a detailed diagnostic is printed. For clients which use L<Test::More>,
+this is the one of the three functions (L</is_same_sql_bind>, L</is_same_sql>,
+L</is_same_bind>) that needs to be imported.
 
 =head2 is_same_sql
 
-  is_same_sql(
-    $given_sql,
-    $expected_sql,
-    $test_msg
-  );
+  is_same_sql($given_sql, $expected_sql, $test_msg);
 
-Compares given and expected SQL statements via L</eq_sql>, and calls
-L<Test::Builder/ok> on the result, with C<$test_msg> as message.
-If the test fails, a detailed diagnostic is printed.
+Compares given and expected SQL statements, and calls L<Test::Builder/ok> on
+the result, with C<$test_msg> as message. If the test fails, a detailed
+diagnostic is printed. For clients which use L<Test::More>, this is the one of
+the three functions (L</is_same_sql_bind>, L</is_same_sql>, L</is_same_bind>)
+that needs to be imported.
 
 =head2 is_same_bind
 
-  is_same_bind(
-    \@given_bind,
-    \@expected_bind,
-    $test_msg
-  );
+  is_same_bind(\@given_bind, \@expected_bind, $test_msg);
 
-Compares given and expected bind values via L</eq_bind>, and calls
-L<Test::Builder/ok> on the result, with C<$test_msg> as message.
-If the test fails, a detailed diagnostic is printed.
+Compares given and expected bind values, and calls L<Test::Builder/ok> on the
+result, with C<$test_msg> as message. If the test fails, a detailed diagnostic
+is printed. For clients which use L<Test::More>, this is the one of the three
+functions (L</is_same_sql_bind>, L</is_same_sql>, L</is_same_bind>) that needs
+to be imported.
 
 =head2 eq_sql_bind
 
-  my $is_same = eq_sql_bind(
-    $given_sql, \@given_bind,
-    $expected_sql, \@expected_bind,
-  );
+  my $is_same = eq_sql_bind($given_sql,    \@given_bind,
+                            $expected_sql, \@expected_bind);
 
-  my $is_same = eq_sql_bind(
-    \[$given_sql, @given_bind],
-    \[$expected_sql, @expected_bind],
-  );
-
-  my $is_same = eq_sql_bind(
-    $dbic_rs->as_query
-    $expected_sql, \@expected_bind,
-  );
-
-Unpacks C<@_> depending on the given arguments and calls L</eq_sql> and
-L</eq_bind>, returning their combined result.
+Compares given and expected pairs of C<($sql, \@bind)>. Similar to
+L</is_same_sql_bind>, but it just returns a boolean value and does not print
+diagnostics or talk to L<Test::Builder>.
 
 =head2 eq_sql
 
@@ -453,13 +358,14 @@ When L</eq_sql> returns false, the global variable
 C<$sql_differ> contains the SQL portion
 where a difference was encountered.
 
+
 =head1 SEE ALSO
 
 L<SQL::Abstract>, L<Test::More>, L<Test::Builder>.
 
 =head1 AUTHORS
 
-Laurent Dami <laurent.dami AT etat  geneve  ch>
+Laurent Dami, E<lt>laurent.dami AT etat  geneve  chE<gt>
 
 Norbert Buchmuller <norbi@nix.hu>
 
